(* strman.pas -- StrMan string manipulation library

   This file is part of the StrMan string manipulation library.

   Copyright (c) 1999 Aleksey V. Vaneev

   The StrMan library is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License, or (at your option) any later version.

   The StrMan library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with the StrMan library; see the file COPYING.
   If not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Aleksey V. Vaneev
   <picoder@sbis.komi.ru>
   http://sbis.komi.ru/picoder
 *)

UNIT StrMan;

INTERFACE

(*
 * CharReplicate
 * --------------------------------------------------------------------------
 * eng: This function returns a string with Character replicated Count
 *      times.
 * --------------------------------------------------------------------------
 * rus:  㭪 頥 ப,   Count ७
 *      ᨬ Character.
 * --------------------------------------------------------------------------
 * MyString := CharReplicate ('A', 64);
 *)

FUNCTION CharReplicate (Character: Char; Count: Byte): String;

(*
 * StringReplicate
 * --------------------------------------------------------------------------
 * eng: This function returns a string with Source string replicated Count
 *      times.
 * --------------------------------------------------------------------------
 * rus:  㭪 頥 ப,   Count ७
 *      ப Source.
 * --------------------------------------------------------------------------
 * MyString := StringReplicate ('Hi! ', 10);
 *)

FUNCTION StringReplicate (Source: String; Count: Byte): String;

(*
 * CharPos
 * --------------------------------------------------------------------------
 * eng: This function searches for a character CharToFind in a string Source
 *      starting at character Offs (counts from zero). Returns nonzero
 *      position of occurence, or zero if character was not found.
 * --------------------------------------------------------------------------
 * rus:  㭪  ᨬ CharToFind  ப Source, 稭 
 *      ᨬ Offs (   ). 頥 㫥 
 *       ப, ᫨ ᨬ  ,  ᫨ ᨬ   -
 *      頥 .
 * --------------------------------------------------------------------------
 * position := CharPos ('a', 'This is a test string', 0);
 *)

FUNCTION CharPos (CharToFind: Char; Source: String; Offs: Byte): Byte;

(*
 * StringPos
 * --------------------------------------------------------------------------
 * eng: This function searches for a substring StringToFind in a string
 *      Source starting at Offs character (counts from zero). Returns
 *      nonzero position of occurence, or zero if specified substring was
 *      not found.
 * --------------------------------------------------------------------------
 * rus:  㭪  ப StringToFind  ப Source, 稭 
 *      ᨬ Offs (   ). 頥 㫥 
 *       ப, ᫨ ப  ,  ᫨   - 頥
 *      .
 * --------------------------------------------------------------------------
 * position := StringPos ('test', 'This is a test string', 0);
 *)

FUNCTION StringPos (StringToFind: String; Source: String; Offs: Byte): Byte;

(*
 * StringJustifyRight, StringJustifyLeft, StringJustifyCenter
 * --------------------------------------------------------------------------
 * eng: This set of functions can be used to justify string Source to
 *      specified position (right, left or center) and pad it with
 *      characters JustifyChar to form a string of Width length. If length
 *      of Source is larger than the specified Width truncation will take
 *      place.
 * --------------------------------------------------------------------------
 * rus:   㭪権   ᯮ짮  ࠢ ப
 *      Source    ࠢ ,   業. ࠢ
 *      ⢫   ᨬ JustifyChar. ⥫쭠 ப
 *      㤥   Width. ᫨  ப Source  Width, 
 *      ந  祭.
 * --------------------------------------------------------------------------
 * OutString := StringJustifyRight ('Item1', 40, '.');
 * OutString := StringJustifyLeft ('Item1', 40, '.');
 * OutString := StringJustifyCenter ('Item1', 40, '.');
 *)

FUNCTION StringJustifyRight (Source: String; Width: Byte; JustifyChar: Char): String;
FUNCTION StringJustifyLeft (Source: String; Width: Byte; JustifyChar: Char): String;
FUNCTION StringJustifyCenter (Source: String; Width: Byte; JustifyChar: Char): String;

(*
 * StringJustifyWrap
 * --------------------------------------------------------------------------
 * eng: This function can be used to expand the string to the requested
 *      Width. Necessary amount of chars JustifyChar is added between the
 *      words of the string which is delimited to words with characters
 *      contained in WordDelimiters string (one or more continuous
 *      delimiting characters in a string is processed as a single
 *      delimiter). If specified Width is less than the length of the
 *      string, the string will not be truncated.      
 * --------------------------------------------------------------------------
 * rus:  㭪   ᯮ짮  ७ ப 
 *      室  Width. 㭪 ࠢ୮  室
 *      ᫮ ᨬ JustifyChar  ᫮ ப,  ࠧ
 *       ᫮ ᨬ, ᮤঠ騬  ப WordDelimiters (᪮쪮
 *      ࠧ⥫ ᨬ  ࠡ뢠 ப 
 *      ࠡ뢠  ). ᫨ 㪠  Width , 祬
 *       ᠬ ࠡ뢠 ப, ப  㤥 祭.
 * --------------------------------------------------------------------------
 * OutString := StringJustifyWrap ('This is a JustifyWrap', 40, ' ', '.');
 *)

FUNCTION StringJustifyWrap (Source: String; Width: Byte; WordDelimiters: String; JustifyChar: Char): String;

(*
 * StringTrimRight, StringTrimLeft, StringTrimAll
 * --------------------------------------------------------------------------
 * eng: This set of functions can be used to remove all unwanted leading or
 *      trailing characters, or both. All unwanted characters specified in
 *      Garbage string. Common use of these functions is a trimming of
 *      space and tab characters before some kind of text parsing.
 * --------------------------------------------------------------------------
 * rus:   㭪権   ᯮ짮  㤠 
 *      ⥫ ᨬ  砫 ப,     ६
 *        砫,    . ,  室 ,
 *      㪠뢠  ப Garbage. 筮  㭪樨 ਬ 
 *      㤠 ᨬ "஡"  "⠡" । - ᪮
 *      ࠡ⪮ ⥪⮢ .
 * --------------------------------------------------------------------------
 * OutString := StringTrimRight ('   a string for test     ', ' ');
 * OutString := StringTrimLeft ('   a string for test     ', ' ');
 * OutString := StringTrimAll ('   a string for test     ', ' ');
 *)

FUNCTION StringTrimRight (Source: String; Garbage: String): String;
FUNCTION StringTrimLeft (Source: String; Garbage: String): String;
FUNCTION StringTrimAll (Source: String; Garbage: String): String;

(*
 * StringReverse
 * --------------------------------------------------------------------------
 * eng: This function reverses string Source, so that string 'today' will
 *      look 'yadot'.
 * --------------------------------------------------------------------------
 * rus:  㭪 "ॢ稢" ப Source ⠪, ,  ਬ,
 *      ப 'today' 㤥 룫拉  'yadot'.
 * --------------------------------------------------------------------------
 * OutString := StringReverse ('today');
 *)

FUNCTION StringReverse (Source: String): String;

(*
 * StringFromData
 * --------------------------------------------------------------------------
 * eng: This function retrieves a pascal-string from untyped data.
 * --------------------------------------------------------------------------
 * rus:  㭪  ப  ⨯஢ .
 * --------------------------------------------------------------------------
 * OutString := StringFromData (MyDataPtr, 255);
 *)

FUNCTION StringFromData (Source: Pointer; Count: Byte): String;

(*
 * StringFromNul
 * --------------------------------------------------------------------------
 * eng: This function retrieves a pascal-string from ASCIIZ string which
 *      ends with 0 character. If length of ASCIIZ string is more than 255
 *      characters truncation will take place.
 * --------------------------------------------------------------------------
 * rus: 㭪  ப  ASCIIZ ப,  稢
 *      ᨬ 0. ᫨  ASCIIZ ப  255 ᨬ, ⠫
 *      ᨬ  ⪨.
 * --------------------------------------------------------------------------
 * OutString := StringFromNul (StrPtr);
 *)

FUNCTION StringFromNul (Source: Pointer): String;

(*
 * StringCase
 * --------------------------------------------------------------------------
 * eng: This functions recodes string Source using 256-byte table
 *      RecodeTable. Common use for this function is upper- or lowercasing
 *      of strings.
 * --------------------------------------------------------------------------
 * rus:  㭪 ४ ப Source, ᯮ 256-⮢
 *      ⠡ RecodeTable. 筮  㭪 室 
 *      ஢ ப  孨   ॣ,  ⠪
 *        ஢ (ਬ,  cp866  win1251).
 * --------------------------------------------------------------------------
 * OutString := StringCase ('This is a test', @UpperCase);
 *)

FUNCTION StringCase (Source: String; RecodeTable: Pointer): String;

(*
 * StringExtract
 * --------------------------------------------------------------------------
 * eng: This functions extracts Count-sized substring from a string Source,
 *      starting at offset Offs (counting from zero). If Offs is out of
 *      range, empty string will be returned. If specified Count is larger
 *      than the actual data present, it will be automatically decreased
 *      to correct amount.
 * --------------------------------------------------------------------------
 * rus:  㭪  ப  Count ᨬ  ப
 *      Source, 稭  樨 Offs (   ). ᫨ 㪠
 *      Offs   , 㤥 饭  ப. ᫨
 *      㪠 Count 訩, 祬    ᠬ ,  㤥
 *      ⮬᪨ 㬥襭  ⨬ 稭.
 * --------------------------------------------------------------------------
 * OutString := StringExtract ('This is a test', 0, 4);
 *)

FUNCTION StringExtract (Source: String; Offs: Byte; Count: Byte): String;

(*
 * StringRemove
 * --------------------------------------------------------------------------
 * eng: This functions removes Count characters from string Source starting
 *      at Offs character (counts from zero). If specified Offs is out of
 *      range, no removing will take place.
 * --------------------------------------------------------------------------
 * rus:  㭪 㤠 Count ᨬ  ப Source, 稭 
 *      樨 Offs (   ). ᫨ 㪠 Offs 
 *      , १ 㤥 ࠢ Source.
 * --------------------------------------------------------------------------
 * OutString := StringRemove ('This is a test', 0, 5);
 *)

FUNCTION StringRemove (Source: String; Offs: Byte; Count: Byte): String;

(*
 * StringInsert
 * --------------------------------------------------------------------------
 * eng: This functions inserts substring SubString to string Source at Offs
 *      character (counts from zero). If specified Offs is out of range,
 *      substring will be appended to the end of the string.
 * --------------------------------------------------------------------------
 * rus:  㭪 ⠢ ப SubString  ப Source  
 *      Offs (   ). ᫨ 㪠 Offs  ,
 *      ப 㤥 ᮥ.
 * --------------------------------------------------------------------------
 * OutString := StringInsert ('is ', 'This a test', 5);
 *)

FUNCTION StringInsert (SubString: String; Source: String; Offs: Byte): String;

(*
 * StringWordCount, StringWordGet, StringWordPos
 * --------------------------------------------------------------------------
 * eng: This set of functions can be used for extraction of single words (or
 *      tokens) from a string Source delimited with characters contained in
 *      a string WordDelimiters. (If string Source contains places with
 *      several delimiters together they will be processed as a single
 *      delimiter. If string has leading or trailing delimiters they will be
 *      ignored during processing). Functions "Get" and "Pos" also have a
 *      Num parameter (counts from 1) that specifies word that is needed.
 *
 *      StringWordGet returns empty string if specified word number is out
 *      of range.
 *
 *      StringWordPos returns zero if specified word number is out of range.
 * --------------------------------------------------------------------------
 * rus:   㭪権   ᯮ짮  祭 ⤥
 *      ᫮  ப Source, ࠧ  ᫮ ᨬ, ᮤঠ騬
 *       ப WordDelimiters. ( ப Source ᪮쪮 ࠧ⥫
 *      ᨬ   ࠡ⪥ ꥤ  . ᫨  ப
 *      ࠧ⥫   砫 /   , 
 *      ). 㭪樨 "Get"  "Pos", ஬ ⮣,  ࠬ
 *      Num (   1),  㪠뢠  室 ᫮.
 *
 *      㭪 StringWordGet 頥  ப, ᫨ 㪠 ᫮
 *       .
 *
 *      㭪 StringWordPos 頥 , ᫨ 㪠 ᫮ 
 *      .
 * --------------------------------------------------------------------------
 * WordCount := StringWordCount ('cmd1; comment; execstr', '; ');
 * WordString := StringWordGet ('cmd1; comment; execstr', '; ', 1);
 * WordPosition := StringWordPos ('cmd1; comment; execstr', '; ', 3);
 *)

FUNCTION StringWordCount (Source: String; WordDelimiters: String): Byte;
FUNCTION StringWordGet (Source: String; WordDelimiters: String; Num: Byte): String;
FUNCTION StringWordPos (Source: String; WordDelimiters: String; Num: Byte): Byte;

(*
 * StringMatching
 * --------------------------------------------------------------------------
 * eng: This function can be used to test a string for matching with the
 *      specified pattern. Pattern can contain '?' and '*' characters. '?'
 *      means there can be any character in a string, '*' means there can be
 *      any amount or none of any characters in a string.
 * --------------------------------------------------------------------------
 * rus:  㭪   ᯮ짮  ஢ન ப 
 *      ᮮ⢥⢨  ᪥. ᪠  ᮤঠ ᨬ '?' 
 *      '*'.  '?'  ᪥ 砥,   ⮬   ஢塞
 *      ப    ᨬ.  '*'  ப 砥,  
 *      ⮣   ஢塞 ப    ᫮ (  
 *      ᮢᥬ)  ᨬ.
 * --------------------------------------------------------------------------
 * Matching := StringMatching ('This is a test', '*is*');
 *)

FUNCTION StringMatching (InString: String; Pattern: String): Boolean;

(* LowerCase
 * --------------------------------------------------------------------------
 * eng: This table can be used to lower case all uppercase letters in a
 *      string. Be cautious - this table also lowers all russian letters
 *      in code page 866. If you don't need that, use LowerCaseEng instead.
 * --------------------------------------------------------------------------
 * rus:  ⠡   ᯮ짮  ॢ  ய
 *      㪢  . ᪨ 㪢  ਢ  ஢ cp866.
 *)

CONST
	LowerCase: ARRAY [0..255] OF Byte = (
		0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17,
		18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33,
		34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49,
		50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 97,
		98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110,
		111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 91,
		92, 93, 94, 95, 96, 97, 98, 99, 100, 101, 102, 103, 104, 105,
		106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118,
		119, 120, 121, 122, 123, 124, 125, 126, 127, 160, 161, 162, 163,
		164, 165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 175, 224,
		225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 237,
		238, 239, 160, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170,
		171, 172, 173, 174, 175, 176, 177, 178, 179, 180, 181, 182, 183,
		184, 185, 186, 187, 188, 189, 190, 191, 192, 193, 194, 195, 196,
		197, 198, 199, 200, 201, 202, 203, 204, 205, 206, 207, 208, 209,
		210, 211, 212, 213, 214, 215, 216, 217, 218, 219, 220, 221, 222,
		223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235,
		236, 237, 238, 239, 240, 241, 242, 243, 244, 245, 246, 247, 248,
		249, 250, 251, 252, 253, 254, 255);

(* UpperCase
 * --------------------------------------------------------------------------
 * eng: This table can be used to upper case all lowercase letters in a
 *      string. Be cautious - this table also uppers all russian letters
 *      in code page 866. If you don't need that, use UpperCaseEng instead.
 * --------------------------------------------------------------------------
 * rus:  ⠡   ᯮ짮  ॢ  
 *      㪢  ய. ᪨ 㪢  ਢ  ஢ cp866.
 *)

CONST
	UpperCase: ARRAY [0..255] OF Byte = (
		0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17,
		18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33,
		34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49,
		50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65,
		66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81,
		82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 65,
		66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81,
		82, 83, 84, 85, 86, 87, 88, 89, 90, 123, 124, 125, 126, 127,
		128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140,
		141, 142, 143, 144, 145, 146, 147, 148, 149, 150, 151, 152, 153,
		154, 155, 156, 157, 158, 159, 128, 129, 130, 131, 132, 133, 134,
		135, 136, 137, 138, 139, 140, 141, 142, 143, 176, 177, 178, 179,
		180, 181, 182, 183, 184, 185, 186, 187, 188, 189, 190, 191, 192,
		193, 194, 195, 196, 197, 198, 199, 200, 201, 202, 203, 204, 205,
		206, 207, 208, 209, 210, 211, 212, 213, 214, 215, 216, 217, 218,
		219, 220, 221, 222, 223, 144, 145, 146, 147, 148, 149, 150, 151,
		152, 153, 154, 155, 156, 157, 158, 159, 240, 241, 242, 243, 244,
		245, 246, 247, 248, 249, 250, 251, 252, 253, 254, 255);

(* LowerCaseEng
 * --------------------------------------------------------------------------
 * eng: This table can be used to lower case all uppercase letters in a
 *      string. Only english characters are affected.
 * --------------------------------------------------------------------------
 * rus:  ⠡   ᯮ짮  ॢ  ய
 *      㪢  .   ⮫쪮  ᪨ 㪢.
 *)

CONST
	LowerCaseEng: ARRAY [0..255] OF Byte = (
		0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17,
		18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33,
		34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49,
		50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 97,
		98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110,
		111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 91,
		92, 93, 94, 95, 96, 97, 98, 99, 100, 101, 102, 103, 104, 105,
		106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118,
		119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 131,
		132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 144,
		145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157,
		158, 159, 160, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170,
		171, 172, 173, 174, 175, 176, 177, 178, 179, 180, 181, 182, 183,
		184, 185, 186, 187, 188, 189, 190, 191, 192, 193, 194, 195, 196,
		197, 198, 199, 200, 201, 202, 203, 204, 205, 206, 207, 208, 209,
		210, 211, 212, 213, 214, 215, 216, 217, 218, 219, 220, 221, 222,
		223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235,
		236, 237, 238, 239, 240, 241, 242, 243, 244, 245, 246, 247, 248,
		249, 250, 251, 252, 253, 254, 255);

(* UpperCaseEng
 * --------------------------------------------------------------------------
 * eng: This table can be used to upper case all lowercase letters in a
 *      string. Only english characters are affected.
 * --------------------------------------------------------------------------
 * rus:  ⠡   ᯮ짮  ॢ  
 *      㪢  ய.   ⮫쪮  ᪨ 㪢.
 *)

CONST
	UpperCaseEng: ARRAY [0..255] OF Byte = (
		0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17,
		18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33,
		34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49,
		50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65,
		66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81,
		82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 65,
		66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81,
		82, 83, 84, 85, 86, 87, 88, 89, 90, 123, 124, 125, 126, 127,
		128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140,
		141, 142, 143, 144, 145, 146, 147, 148, 149, 150, 151, 152, 153,
		154, 155, 156, 157, 158, 159, 160, 161, 162, 163, 164, 165, 166,
		167, 168, 169, 170, 171, 172, 173, 174, 175, 176, 177, 178, 179,
		180, 181, 182, 183, 184, 185, 186, 187, 188, 189, 190, 191, 192,
		193, 194, 195, 196, 197, 198, 199, 200, 201, 202, 203, 204, 205,
		206, 207, 208, 209, 210, 211, 212, 213, 214, 215, 216, 217, 218,
		219, 220, 221, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231,
		232, 233, 234, 235, 236, 237, 238, 239, 240, 241, 242, 243, 244,
		245, 246, 247, 248, 249, 250, 251, 252, 253, 254, 255);

IMPLEMENTATION

{$IFDEF VIRTUALPASCAL}
	{$I STRMAN.PA2}
{$ELSE}
	{$IFDEF FPC}
		{$I STRMAN.PA0}
	{$ELSE}
		{$IFDEF VER70}
			{$I STRMAN.PA1}
		{$ELSE}
			{$IFDEF __TMT__}
				{$I STRMAN.PA2}
			{$ELSE}
				{$IFDEF WIN32}
					{$I STRMAN.PA0}
				{$ELSE}
					{$IFDEF VER80}
						{$I STRMAN.PA1}
					{$ELSE}
						{$I STRMAN.PA0}
					{$ENDIF}
				{$ENDIF}
			{$ENDIF}
		{$ENDIF}
	{$ENDIF}
{$ENDIF}

END.
